function [varioe, variot, Paramv, Cost] = Estim_Variogram_Fit(locs, RF, vopt)
%   Estim_Fit_Variogram_Classical performs the following operations:
%   1. It loads a data set consisting of X, Y (location) and RF (data) values.
%   2. It estimates the empirical variogram
%   3. It performs a weighted least-squares fit to find the optimal
%   variogram model among the five families: Expo, Gaus, Sphe, Mate, Power
%
%================ INPUT PARAMETERS ========================================
%   LOCS:       Array containing the coordinates of points ( N x 1)
%   RF:         Data values ( N x K) If K >1 the m
%   VOPT.NE:    (Optional ) Number of lag bin edges used to contruct
%               the variogram (default value = 25). Number of distance
%               classes  Nc = Ne - 1;
%   VOPT.SC:    (Optional) Fraction of maximum distance to be used in
%               variogram estimation to determine max lag distance
%   VOPT.MODEL: (Optional ) If you need to run ony one of the models to
%               reduce time, then define it here. Options are: 'Gaus',
%               'Expo', 'Sphe', 'Mate', 'Power'
%   VOPT.PARI:  (Optional) Initial estimates for the variogram parameters
%
%================ OUTPUT PARAMETERS =======================================
%   VARIOE:             Structure array containing  empirical variogram
%   VARIOE.LAGS:        Array of lag distances ( Nc x 1)
%   VARIOE.VARIOZ:      Array of empirical variogram values ( Nc x 1)
%   VARIOE.NPAIRS:      Number of pairs per lag distance ( Nc x 1)
%   VARIOT:             Structure array containing  theoretical variogram
%   VARIOT.MODEL:       Optimal variogram model
%   VARIOT.LAGS:        Array of lag distances for SSRF variogram ( Nc x 1)
%   VARIOT.VARIO:       Array of model variogram values ( Nc x 1)
%   PARAMV:             Parameter vector [C0 C1 XI {NU}] 
%                       C0: nugget term variance
%                       C1: partial sill (correlated term variance)
%                       XI: Correlation length
%                       NU: Smoothness index (only for Matern model)
%                       For SSRF models: [C0 ETA0 ETA1 XI]
%   COST:               Cost of the variogram fit
%
%=======================VARIOGRAM MODELS INCLUDED==========================
%	'Gaus':	Gaussian:	sigma^2 * [1-exp(-r^2/xi^2)]
%	'Expo':	Exponential: sigma^2 * { 1-exp[- sqrt(r/xi) ] }
%   'Sphe': Spherical: sigma^2 *(1.5*r/xi-0.5*(r/xi).^3).*(r<=xi);
%   'Mate': Matern:  1-(2^(1-nu)/gamma(nu))*((ka*r)^(nu))*BESSELK(nu,ka*r);
%   'Power':Algebraic:  a* r^(2H)
%
%========================= FUNCTIONS USED==================================
%
%   This script requires the following functions:
%   WLS_WEIGHTED, VARIO_EMPIR_D, THEOR_VARIOGRAMS_CLASSICAL
%
%==========================================================================
%  Copyright (C) 2019  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%
%========================= EXAMPLE ========================================
%
%   close all; load data_vario_East.mat;
%   locs = data_vario_East(:,1); z = data_vario_East(:,2);
%   [varioe, variot, Paramv, ~] = Estim_Variogram_Fit(locs, z);
%

% dd = size(locs,2);
global dd

%   Set up the model suite
model{1} = 'Gaus';
model{2} = 'Expo';
model{3} = 'Sphe';
model{4} = 'Mate';
model{5} = 'Power';
model{6} = 'SSRF';
num_model = 6; 

%==========================================================================
%   NORMALIZE COORDINATES
%==========================================================================

if nargin >2 && isfield(vopt,'Norm')==1
    for i =1 : dd
        temp = locs(:,i);
        locs(:,i) = (temp - mean(temp))/std(temp);
    end
end
clear temp; 


%==========================================================================
%   ESTIMATE EMPIRICAL VARIOGRAM
%==========================================================================
% Determine number of lag classes
if nargin >2 && isfield(vopt,'Ne')==1
    Ne = vopt.Ne;
else
    Ne = 11;
end

% Determine maximum lag using a scaling factor: max lag = max dist/scaled
if nargin >2 && isfield(vopt,'sc')==1
    scaled = vopt.sc;
else 
    scaled = 2; 
end   
    [lags, varioRF, npairs] =  vario_empir_d(dd, locs, RF, Ne, scaled);

varioe.lags = lags;
varioe.varioz = varioRF ; 
varioe.npair = npairs ;

maxd = max(lags); % Maximum distance lag to be used

options = optimset('display','final','TolX', 1e-4, 'TolFun', 1e-4, ...
    'MaxFunEvals', 10000,'MaxIter', 10000);

%==========================================================================
%   DETERMINE THE OPTIMAL VARIOGRAM MODEL USING LEAST SQUARES FIT
%==========================================================================


if nargin >2 && isfield(vopt,'model')==1
    modloop = find(strcmpi(model,vopt.model));
else
    modloop = 1:num_model;
end

%   Initialize vectors
numl = length(modloop);
Cost = zeros(numl, 1); 
Paramv = cell(numl, 1);
variot = cell(numl, 1); 
opts = cell(numl, 1);

for i=modloop  % Loop over all mod+
    mode.model = model{i};
    mode.dd = dd; 
    
%==========================================================================
%   INITIALIZE PARAMETERS
%==========================================================================
if nargin==2 || ~isfield(vopt,'pari') 
if i < 4 
        c0_0 = var(RF)/50;
        xi_0 = 1;
        c1_0 = var(RF);
    
        x0 = [c0_0 c1_0  xi_0]; % Initial parameter estimate
        LB = [0       var(RF)/3 min(lags)];
        UB = [var(RF) 2*var(RF) max(lags)];
    
    elseif i==4  % Matern
        c0_0 = 0;
        xi_0 = 1;
        c1_0 = var(RF);
        nu_0 = 1;
    
        x0 = [c0_0 c1_0  xi_0 nu_0]; % Initial parameter estimate
        LB = [0      var(RF)/2    min(lags)  0.5];
        UB = [var(RF)  2*var(RF) max(lags)  3];
    
    elseif i==5  % Power
        c0_0 = 0;
        a_0 = 1;
        b_0 = 1;
        x0 = [c0_0 a_0  b_0]; % Initial parameter estimate
        LB = [0         min(lags) 0.001];
        UB = [var(RF)/3 max(lags) 1.9999];
    
    elseif i==6  % Spartan
%         [C0 XI C1 {C2}] -> [nugget xi eta_0 eta_1] 
        c0_0 = 0; %nugget
        xi_0 = 1; %xi
        c2_0 = 2; %eta_1
        [varsrf0] = cov_ssrf(dd,eps,[1 c2_0 xi_0],0); %[eta_0 eta_1 xi] -> [C1 C2 XI]
        c1_0 = var(RF)/varsrf0; %eta_0
        
        x0 = [c0_0 c1_0 c2_0 xi_0]; % Initial parameter estimate
%         LB = [eps        var(RF)/2  0.5  min(lags)];
%         UB = [var(RF)/3 2*var(RF)  3   max(lags)];
        LB = [eps eps -1.999 min(lags)];
        UB = [max(RF)/3 2*var(RF) 10^3 max(lags)];
    
    elseif i==7
        disp('Wrong number of models');
        return;
    
end
else  % Initial values set in vopt
    x0 = vopt.pari; 
    if i < 4 
        LB = [0  var(RF)/2 min(lags)];
        UB = [var(RF) 2*var(RF) max(lags)];    
    elseif i==4  % Matern
        LB = [0      var(RF)/2    min(lags)  0.5];
        UB = [var(RF)  2*var(RF) max(lags)  3];
    elseif i==5  % Power
        LB = [0         min(lags) 0.001];
        UB = [var(RF)/3 max(lags) 1.9999];    
    elseif i==6  % Spartan
        LB = [eps eps -1.999 min(lags)];
        UB = [max(RF)/3 2*var(RF) 10^3 max(lags)];    
    elseif i==7
        disp('Wrong number of models');
        return;
    
    end
    
end

 %disp(LB), disp(UB),  disp(x0), disp(mode.model);
% save varioRF varioe mode;

% Optimization

    varioe.lags = lags;
    varioe.varioz = varioRF;
    varioe.npair = npairs;
    

    [x, fval, ~, ~] = fmincon(@(x) wls_weighted(x, mode, varioe, 2), ...
        x0, [], [], [], [],  LB, UB, [], options);

%     [C0 C1 XI {nu}] for every model except
%     x0 = [c0 c1 c2 xi]; for SSRF meaning [nugget eta_0 eta_1 xi]
    Paramv{i} = x;
%     disp(['size(fval):' num2str(size(fval))])
%     disp(['size(Cost):' num2str(size(Cost))])
%     Cost(i) = fval;
    
    variot{i}.model = model{i};

    [ lagsn, variom ] = theor_variograms( model{i}, Paramv{i}, maxd );
    variot{i}.lags = lagsn;
    variot{i}.vario = variom;
end

%   Remove empty cells from cell array of estimated parameters
Paramv = Paramv(~cellfun('isempty',Paramv)) ; 
variot = variot(~cellfun('isempty',variot)) ; 

