function [weights, zhat, zstd]= GPR(input, z, varmodel, output) 

%   GPR returns the GP estimates over a vector of points based  
%   on the mode of the respective marginal PDF. It uses the variogram 
%   in the formulation of the prediction system
%
%================ INPUT PARAMETERS ========================================
%
%   input:      (N x D)  input vector (matrix), N: #sampling pts; D: dimension
%   z:          Process values
%   varmodel:   Structure containing the variogram model (char) and its
%               parameters
%   output:     (P x D) output vector (matrix) of prediction pts

%================ OUTPUT PARAMETERS =======================================
%
%   zhat: Field values at estimation locations
%   zstd:  Standard deviation of OK at estimation locations
%   weights:  GPR weights
%==========================================================================
%
%	Copyright (C) 2022 Dionisis Hristopulos
%   email: dchristopoulos@ece.tuc.gr
% 
%	Last Modified: January 2022
% 
%	This program is free software; you can redistribute it and/or modify it
%	under the terms of the GNU General Public License as published by the
%	Free Software Foundation (version 2 of the License or  later version).
%
%	This program is distributed WITHOUT ANY WARRANTY.
%	See the GNU General Public License for more details.
% 
%==========================================================================

global dd

model = varmodel.model;
Param = varmodel.Param;

% Define the variogram models that can be used in OK

%   Exponential variogram
expov = @(x,pm) pm(1).* (x > eps) + pm(2)* ( 1 - exp(-x/pm(3)) );
%   Gaussian variogram
gausv = @(x,pm) pm(1).* (x > eps) + pm(2)*  ( 1 - exp(- (x/pm(3)).^2) );
%   Spherical variogram
sphev = @(x,pm) pm(1).* (x > eps) + pm(2)* ( ( 3 *(x/pm(3))/2  - ((x/pm(3)).^3)/2 ).* ...
    (x <=pm(3)) + 1 .* (x > pm(3) ) );
%   Matern variogram
matev =  @(x,pm) pm(1).* (x > eps) + pm(2)* (1 - (2^(1-pm(4))/gamma(pm(4))) ...
    * ( (x/pm(3)).^pm(4)) .* besselk(pm(4), (x/pm(3)) ) );
%   Power-law variogram
powev = @(x,pm) pm(1).* (x > eps) + pm(2)* (x .^pm(3));
% Spartan variogram
ssrf = @(x,pm) pm(1).* (x > eps) + (cov_ssrf(dd, eps, pm(2:end), 0) - cov_ssrf(dd, x, pm(2:end), 0));

%==========================================================================

%==========================================================================
%   Setup the Kriging system (based on variogram)
%==========================================================================

Dd = pdist(input, 'euclidean');
rr = squareform(Dd);
sizerr = size(rr);

rrv = reshape(rr, numel(rr), 1);
clear rr;
rrv(rrv==0) = eps;

%   Variogram matrix
switch model
    case 'Expo'
        G = expov(rrv, Param);
    case 'Gaus'
        G = gausv(rrv, Param);
    case 'Sphe'
        G = sphev(rrv, Param);
    case 'Mate'
        G = matev(rrv, Param);
    case 'Powe'
        G = powev(rrv, Param);
    case 'SSRF'
        G = ssrf(rrv, Param);
end
Gmat = reshape(G, sizerr);
Ns = numel(z);  % Number of sampling points
z = z(:)'; %    Ensure that z is a row matrix

%==========================================================================
% For Ordinary kriging
%==========================================================================
Iv1 = ones(1, Ns);
Iv2  = ones(Ns,1);

Gkrig = [ Gmat Iv2; [Iv1 0] ];

%==========================================================================
%   First estimate the thickness at the cell centers
%==========================================================================

P = size(output, 1); % Number of prediction points

zhat = zeros(P,1);

varok = zeros(P,1);

for j =1: P
    
    d0i = pdist2(input, output(j,:));
    d0i(d0i==0) = eps;
%==========================================================================

    % Ordinary kriging solution
    switch model
        case 'Expo'
            vecok = [ expov(d0i, Param); 1];
        case 'Gaus'
            vecok = [ gausv(d0i, Param); 1];
        case 'Sphe'
            vecok = [ sphev(d0i, Param); 1];
        case 'Mate'
            vecok = [ matev(d0i, Param); 1];
        case 'Powe'
            vecok = [ powev(d0i, Param); 1];
        case 'SSRF'
            vecok = [ ssrf(d0i, Param); 1];
    end
    
    weights = Gkrig \vecok;
    ok_weights = weights(1: Ns);
    
    zhat(j) =  z * ok_weights;
    varok(j) =  vecok' * weights;
    
%==========================================================================

end


zstd = sqrt(varok);

zhat = reshape(zhat, P, 1 );
zstd = reshape(zstd, P, 1 );

