%   This script generates a warped sine function contaminated with noise
%   according to the example in Snelson's paper. It samples the function at
%   N points and performs GPR to estimate the function at Ntest points. 
%   The GPR parameters are estimated by variogram fitting for different 
%   variogram models including the Spartan variogram family (see Ref. [3]). 
%   The GPR equations are solved by means of the covariance kernel-based
%   system (function GPR).
%   The script applies both the GPR and wGPR (weighted GPR) algorithms.  
%   Warping is performed using the non-parametric method presented in 
%   Refs. [1], [2] below. 
%==========================================================================
%   FUNCTIONS USED: Estim_Variogram_Fit, wls_weighted, GPR, Gen_Lookup,  
%   KCDF_Estim, Kernel_invNS, cov_ssrf, theor_variograms, vario_empir_d.
%   The functions Gen_Lookup, KCDF_Estim, kde, KernBW, kernel_i, Kernel_invNS 
%   have been implemented by Andreas Pavlides.
%==========================================================================
%
%	Copyright (C) 2022 Dionisis Hristopulos
%   email: dchristopoulos@tuc.gr
% 
%	Last Modified: March 2023
% 
%	This program is free software; you can redistribute it and/or modify it
%	under the terms of the GNU General Public License as published by the
%	Free Software Foundation (version 2 of the License or  later version).
%
%	This program is distributed WITHOUT ANY WARRANTY.
%	See the GNU General Public License for more details.
% 
%==========================================================================
%	REFERENCES 
%==========================================================================
% When using this software please cite the following: 
%
%   1. Agou VD, Pavlides A, Hristopulos DT. Spatial Modeling of 
%   Precipitation Based on Data-Driven Warping of Gaussian Processes. 
%   Entropy. 2022; 24(3):321. https://doi.org/10.3390/e24030321 
%
%   2. Andrew Pavlides, Vasiliki D. Agou, Dionissios T. Hristopulos,
%   Non-parametric kernel-based estimation and simulation of precipitation
%   amount, Journal of Hydrology, Volume 612, Part A, 2022, 127988,
%   https://doi.org/10.1016/j.jhydrol.2022.127988
%
%	3. D. T. Hristopulos, Random Fields for Spatial Data Modeling: 
%   A Primer for Scientists and Engineers. Springer Netherlands, Dordrecht, 2020
%	ISBN  978-94-024-1918-4,
%	doi: 10.1007/978-94-024-1918-414,
%	URL: https://doi.org/10.1007/978-94-024-1918-4_14.
% 

clearvars; close all;
global dd;
dd =1;  % Dimension of test function
% Set the random number generator to the default seed (0) and algorithm (Mersenne Twister)
rng('default')
%==========================================================================
%   Variogram parameters

vopt.Ne = 11;   % Variogram classes
vopt.sc = 2; % Max lag = Max dist / sc ;
zc = 2; %  Critical value of standard normal distribution used for 
%           uncertainty quantification (prediction intervals based on 
%           m +/- zc * Zstd
Modn = 6; % Select the covariance kernel model
%Modn = 6; % Select SSRF model
kernel = 'Tria'; % Kernel used for CDF estimation

switch Modn
    case 1
        modelnam = 'Gaus';
    case 2
        modelnam = 'Expo';
    case 3
        modelnam = 'Sphe';
    case 4
        modelnam = 'Mate';
    case 6
        modelnam = 'SSRF';
end

%   Figure name to be used 
fignam = ['Warp_GP_sine_Cov_' modelnam '_Kernel_' kernel];
%   File name for saving results
filnam = ['Warp_GP_sine_Cov_' modelnam '_Kernel_' kernel];


%==========================================================================
%   Determine the sampling (training) points
N = 100; Ntest = 400;
x = rand(N, 1); 

x = 2*pi*x - pi; 

%   Define the noisy, warped test function
y = sin(x) + 0.1*randn(size(x));
data = nthroot(y, 3); 
figure(1); 
plot(x,data, '*'); 
%==========================================================================
%   Determine the test (validation) points and function values at
%   validation points

xtest = linspace(-pi, pi, Ntest);
xtest = xtest(:); 
ztest = nthroot( sin(xtest), 3); 

%==========================================================================
%%
% Estimate of variogram model for original data
[varioe, variot, Paramv, ~] = Estim_Variogram_Fit(x, data, vopt);

figure(2); 
plot(varioe.lags, varioe.varioz, 'r*');
hold on;
l =cell(7,1);
l{1} = 'Empir';
for j=1:6
    h(j) = plot(variot{j,1}.lags, variot{j,1}.vario, 'LineWidth', 1);
    l{j+1}= variot{j,1}.model;
end
axis tight;
legend(l, 'Location', 'Best');
xlabel('Lags');
ylabel('Variogram');
title('Original Data');
set(gca, 'FontSize', 14);
grid on;
figname = [fignam '_variograms'];
saveas(gcf, figname, 'png');

varmodeli.model = variot{Modn,1}.model; % {2,1} for the exponential , {6,1} for SSRF
varmodeli.Param = Paramv{Modn,1};

[weights, Zhat, Zstd]= GPR(x, data, varmodeli, xtest);
%[weights, Zhat, Zstd]= GPR2(x, z, varmodeli, xtest);

figure(3); 
plot(x, data, 'b.', 'MarkerSize', 10);
hold on; 
plot(xtest, Zhat - zc *Zstd, 'g--', 'LineWidth', 1);
plot(xtest, Zhat + zc *Zstd, 'g--', 'LineWidth', 1);
hgpr=plot(xtest, Zhat, 'm-', 'LineWidth', 1);
htrue=plot(xtest, ztest, 'k-', 'LineWidth', 1);
set(gca, 'FontSize', 14); 
axis tight;
grid on;

%==========================================================================
%   Solve the same problem using warping
% [transform, outdata, time] = warp(data, vopts); 
% normdata = outdata(:,1);    % Transformed normalized data
%==========================================================================

[kcdf, normdata, bgk_band, Supplement] = KCDF_Estim(data, kernel);
[lookup] = Gen_Lookup(data, bgk_band, kernel);  % Generate lookup table

figure(5); 
qqplot(normdata);
axis equal;
axis tight;
grid on;
figname = [fignam '_qqplot_normal_scores'];
saveas(gcf, figname, 'png');

%%
% Estimate of variogram model for original data
[ve_tran, vt_tran, Paramv_tran, ~] = Estim_Variogram_Fit(x, normdata, vopt);

figure(6); 
plot(ve_tran.lags, ve_tran.varioz, 'r*');
hold on;
l =cell(7,1);
l{1} = 'Empir';
for j=1:6
    h(j) = plot(vt_tran{j,1}.lags, vt_tran{j,1}.vario, 'LineWidth', 1);
    l{j+1}= vt_tran{j,1}.model;
end
axis tight;
legend(l, 'Location', 'Best');
xlabel('Lags');
ylabel('Variogram');
title('Transformed Data');
set(gca, 'FontSize', 14);
grid on;
figname = [fignam '_variograms_normscores'];
saveas(gcf, figname, 'png');

v_tran_modeli.model = vt_tran{Modn,1}.model; % {2,1} for the exponential , {6,1} for SSRF
v_tran_modeli.Param = Paramv_tran{Modn,1};

[weights_tran, Zhat_tran, Zstd_tran]= GPR(x, normdata, v_tran_modeli, xtest);
%[weights, Zhat, Zstd]= GPR2(x, z, varmodeli, xtest);

%   Calculate the quantiles of the normal scores
znorm_low = Zhat_tran - zc * Zstd_tran ; 
znorm_high = Zhat_tran + zc * Zstd_tran ; 

%   warp-based Predictions
%[Zhat_warp, ~] = invwarp(transform, Zhat_tran);
[Zhat_warp] = Kernel_invNS(Zhat_tran, lookup );
%   Use quantile invariance under monotonic transformations to obtain the
%   quantiles of the confidence band for the predictive  distribution

% [Zhatwarp_low, ~] = invwarp(transform, znorm_low);
% [Zhatwarp_high, ~] = invwarp(transform, znorm_high);
%%
[Zhatwarp_low] = Kernel_invNS(znorm_low, lookup );
[Zhatwarp_high] = Kernel_invNS(znorm_high, lookup );


%   Plot results of warped warp process 
figure(3); 
hold on;
hwgpr = plot(xtest, Zhat_warp, 'b', 'LineWidth', 1.5);
plot(xtest, Zhatwarp_low, 'c--', 'LineWidth', 1);
plot(xtest, Zhatwarp_high, 'c--', 'LineWidth', 1);
if ~verLessThan('matlab', '9.3')
    set(gca,'defaultAxesTickLabelInterpreter','latex');
    yticklabels(gca, strrep(yticklabels(gca),'-','$-$'));
    xticklabels(gca, strrep(xticklabels(gca),'-','$-$'));   
end
set(gcf, 'color', 'w');

legend([htrue(1), hgpr(1), hwgpr(1) ], 'True','GPR', 'wGPR', 'Location','Best');

figname = [fignam '_GPR_Predictions'];
saveas(gcf, figname, 'png');

%%
%   Generate structure containing the warp predictions for cross-validation
zpwarp.est = Zhat_warp; 
zpwarp.low = Zhatwarp_low; 
zpwarp.high = Zhatwarp_high; 
zpwarp.lev = 100 * ( normcdf(zc) - normcdf(-zc) ); % Level of prediction interval
%==========================================================================






