function [KCDE] = kcde(data, kernel, Nsim)
%   KCDE estimates the CDF of a data vector non-parametrically using the 
%   kernel method described in https://arxiv.org/abs/2109.09961
%
%==========================================================================
%   INPUT
%==========================================================================
%   data:   Vector of data values
%   kernel: Name of kernel function
%   Nsim:   Number of simulated points 
%==========================================================================
%   OUTPUT
%==========================================================================
%   KCDE:   Structure containing the results of the estimation
%   KCDE.h      Botev, Grotowski, Kroese (BGK) bandwidth
%   KCDE.pdf    Kernel density estimate of the PDF
%   KCDE.href   Normal reference rule bandwidth
%   KCDE.step   Empirical step function estimate of the CDF
%   KCDE.cdf	KCDE kernel-based estimate of the CDF
%   KCDE.zval	z values where the kernel CDF is estimated
%   KCDE.table  Lookup table used in inverse transform sampling (ITS)
%   KCDE.simul	Values used at the ITS simulation points
%   KCDE.cdfsim CDF values at the ITS simulation points
%==========================================================================
%   External functions: KernBW_simple, KernSimul
%==========================================================================
%  Kernel choices: 
%       Epanechnikov:   'Epan'
%       Exponential:    'Expo'
%       Triweight:      'Triw'
%       Uniform:        'Unif'
%       Tricubic:       'Tric'
%       BiTriangular:   'Tria'
%       Sphehrical:     'Sphe'
%
%   Bandwidth estimation using the Botev, Grotowski, Kroese rule.
%   Reference:
%   Kernel density estimation via diffusion
%   Z. I. Botev, J. F. Grotowski, and D. P. Kroese (2010)
%   Annals of Statistics, Volume 38, Number 5, pages 2916-2957.
%==========================================================================
%   Written by Andreas Pavlides. Edited by Dionisis Hristopulos. Ocrober
%   2021
%==========================================================================

ndisc = 2^14; % Number of uniformly distributed discretization points of 
%   the interval [zmin, zmax]

%   Minimum and maximum values for the interval on which the density
%   estimate is constructed.  Also, use 
%   >>help kde to see suggested values. 

zmin = eps; % For non-negative valued distributions
zmax = max(data) + 2; 

[h, density, ~, ~] = kde(data, ndisc, zmin, zmax); 
% h is the bandwidth by Botev et al. 

hS=RoT(data);
%   hS is the bandwidth using the Normal Reference Rule. 

KCDE.h = h;
KCDE.pdf = density; 
KCDE.href = hS; % Normal reference rule bandwdith

%% CDF (Staircase and Kernel-derived) 

[ StpFun, CDF_Kern, zval ] = KernBW_simple(data, kernel, h, 0);

KCDE.step = StpFun;
KCDE.cdf = CDF_Kern;
KCDE.zval = zval ; 

%% Simulation using ITS

[ Y, SimProb, LUT] = KernSimul( data, h, kernel, Nsim, rng('shuffle') );

KCDE.table = LUT; 
KCDE.simul = Y;
KCDE.cdfsim = SimProb;



