function [ lags, variom ] = theor_variograms( modelnam, Param, maxd )
%   This function estimates the theoretical variogram for a specified model
%   and parameter values
%================ INPUT PARAMETERS ========================================
%   MODELNAM:   String specifying theoretical variogram model
%   PARAM:      Parameter vector [C0 C1 XI  {nu}] for every model except
%               [c0 c1 c2 xi] for SSRF meaning [nugget eta_0 eta_1 xi] 
%               for the power law the parameters are [c0 a H] where
%               gamma(r)=c0 + a* r^2H
%               Nested models have the following parameter vector: 
%               [c0 c1_1 xi1 (nu1) c1_2 xi2 (nu2)]. 
%               Nesting is implemented for exponential, Gaussian,
%               spherical, Matern models
%   MAXD:       Maximum lag distance     
%
%================ OUTPUT PARAMETERS =======================================
%   LAGS:     Array of lag distances ( 100 x 1)
%   VARIOM:   Array of theoretical variogram values ( 100 x 1)
%
%=======================VARIOGRAM MODELS INCLUDED==========================
%	'Gaus':	Gaussian:	sigma^2 * [1-exp(-r^2/xi^2)]
%	'Expo':	Exponential: sigma^2 * { 1-exp[- sqrt(r/xi) ] }
%   'Sphe': Spherical: sigma^2 *(1.5*r/xi-0.5*(r/xi).^3).*(r<=xi);
%   'Mate': Matern:  1-(2^(1-nu)/gamma(nu))*((ka*r)^(nu))*BESSELK(nu,ka*r);
%   'Power': Algebraic:  a* r^(2H)
%   'SSRF': Spartan: 
%==========================================================================
%  Copyright (C) 2019  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
global dd
lags = linspace(0, maxd); 

switch modelnam 

    case 'Expo'
        variom = Param(2) *  ( 1 - exp(-lags/Param(3)) ); 
        
    case 'Gaus'
        variom = Param(2) *  ( 1 - exp(- (lags/Param(3)).^2) ); 

    case 'Sphe'
        h = lags/Param(3); 
        variom =  Param(2) * ( ( 3 *h/2  - (h.^3)/2 ).* (h <=1) + 1 .* (h>1) );

    case 'Mate'
        h = lags/Param(3); 
        nu = Param(4); 
        cb1 = 2^(1-nu)/gamma(nu);
        variom =  Param(2) * ( 1 -  cb1 * (h.^nu) .* besselk(nu, h) );   

    case 'Power'
        variom = Param(2) * (lags).^Param(3);
        
    case 'Expo-nested'
        variom = Param(2) *  ( 1 - exp(-lags/Param(3)) ) + ...
                 Param(4) *  ( 1 - exp(-lags/Param(5)) ); 
        
    case 'Gaus-nested'
        variom = Param(2) *  ( 1 - exp(- (lags/Param(3)).^2) ) + ...
                Param(4) *  ( 1 - exp(- (lags/Param(5)).^2) ); 

    case 'Sphe-nested'
        h1 = lags/Param(3); 
        h2 = lags/Param(5);
        variom =  Param(2) * ( ( 3 *h1/2  - (h1.^3)/2 ).* (h1 <=1) + ...
                                1 .* (h1>1) ) + ....
                  Param(4) * ( ( 3 *h2/2  - (h2.^3)/2 ).* (h2 <=1) + ...
                                1 .* (h2>1) )  ;

    case 'Mate-nested'
        h1 = lags/Param(3); 
        nu1 = Param(4); 
        cb1 = 2^(1-nu1)/gamma(nu1);
        
        h2 = lags/Param(6); 
        nu2 = Param(7); 
        cb2 = 2^(1-nu2)/gamma(nu2);
        variom =  Param(2)*( 1 -  cb1 * (h1.^nu1) .* besselk(nu1, h1) ) + ...
            Param(5)*( 1 -  cb2 * (h2.^nu2) .* besselk(nu2, h2) );   

    case 'SSRF'
        Paramssrf = Param(2:4);
        [varsrf] = cov_ssrf(dd, eps, Paramssrf, 0);
        [covsrf] = cov_ssrf(dd, lags, Paramssrf, 0);
        variom = varsrf - covsrf;
        
end
variom = variom + Param(1); 
   
    
end

