function [lags,variog,np] = vario_empir_d(dd, locs, z, Ne, scalemaxd)
%   VARIO_EMPIR_D Calculates the omnidirectional experimental variogram for
%   scalar data in d spatial dimensions
%
%================ INPUT PARAMETERS ========================================
%   DD:         Dimension index (integer)
%   LOCS:       Array containing the coordinates of points ( N x DD)
%   Z:          Data values 
%   NE:         Number of lag bin edges used to contruct the variogram 
%               (optional, default value = 25). Number of distance classes 
%               Nc = Ne - 1; 
%   
%================ OUTPUT PARAMETERS =======================================
%   LAGS:       Array of lag distances ( Nc x 1)
%   VARIOG:     Array of variogram values ( Nc x 1)
%   NP:         Number of pairs per lag distance ( Nc x 1)
%
%========================= EXAMPLE ========================================
%
%   [x,rf]=rf1dlu(500,3,1,10);  [lags,variog,np] = vario_empir_d(1, x, rf);
%   figure; plot(lags,variog,'ro');
%
%========================= EXAMPLE ========================================

%  Copyright (C) 2014  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%   This software bundle also includes code developed by other people. 
%
%   DISTMAT: Developed by Joseph Kirk
%
%==========================================================================
N = length(locs);

if size(locs,2) ~= dd
    locs = locs'; 
end
 
if N <=30
    opt=1;
elseif (30< N) && (N<=300)
    opt=2;
else
    opt=3;
end

%   Calculate the distance matrix

switch dd
    case rem(dd,1)~=0
        disp('Non-integer dimension');
        return;
    case dd<0
        disp('Negative dimension');
        return;
    case 2
        tr = distmat(locs,opt);
    case 1
        xs=locs;
        tr = abs(xs*ones(1,N)-ones(N,1)*xs');
    otherwise
        tr = distmat(locs,opt);
end;


ss = (z*ones(1,N)-ones(N,1)*z').^2;
MM = max(max(tr));

% Number of lag bin edges used for computing the empirical semivariogram
if nargin==3
    Ne = 25; 
end

Nc = Ne - 1; %   Number of lag classes

if ~exist('scalemaxd','var')
	scalemaxd = 2;
end
if scalemaxd == 0
	scalemaxd = 2;
end

ur = linspace(0, MM/scalemaxd, Ne);
lags = (ur(2:length(ur)) + ur(1:length(ur)-1))/2;
ur = lags;
step = ur(2)-ur(1);
 
variog = zeros(Nc,1); 
np = zeros(Nc,1); 

for i=1:Nc
    lag_ind = find(( ur(i) - step ) < tr & tr < ( ur(i) + step ) );
    if isempty(lag_ind)
        variog(i) = 0;
        np(i)=0;
    else
        dz2 = ss(lag_ind);
        np(i)= length(lag_ind);    %number of points in each lag
        %y(i)=0*(length(lag_ind)==0)+sum(dz2)/(2*length(lag_ind)+(length(lag_ind)==0));
        variog(i)=sum(dz2)/(2*length(lag_ind));
    end
end

lags = lags(:); 
variog = variog(:); 
np = np(:)/2; %divide by 2, because pairs appear twice as i,j and j,i
