%	WLS_WEIGHTED computes the WLS distance between the empirical variogram 
%	and the THEORETICAL variogram model. 
%   WLS_WEIGHTED can weight the variogram differences
%   (DIfferent weighting schemes can be used.)
%
%   INPUT
%==========================================================================
%   Param:  Model Variogram parameters 
%           SSRF: [ nugget eta0 eta1 xi]
%           EXPO, GAUS, SPHE: [ nugget c1 xi ]
%           MATE: [ nugget c1 xi nu ]  
%           POWE: [ nugget alpha H]
%   MODEL:      Structure containing information about the theoretical model
%   MODEL.dd:     Dimensionality (dd=1, 2, 3)
%   MODEL.NAM:  Variogram type ('SSRF', 'EXPO', 'GAUS', 'SPHE', 'POWE'
%   VARIOE:             Structure field containing the variogram information
%   VARIOE.LAGS:        Array of lag distances ( Nc x 1)
%   VARIOE.VARIORF:     Array of empirical variogram values ( Nc x 1)
%   VARIOES.NPAIRS:      Number of pairs per lag distance ( Nc x 1)
%   weighting:          1 Number of pairs. 2 inverse of square empirical variogram
%                       0 Just least squares are used
%==========================================================================
%	OUTPUT:
%==========================================================================
%	Y:		Value of objective function
%   [y] = wls_weighted(Param, varioe, model, weighting)
%
%==========================================================================
%
%  Copyright (C) 2021  Dionisis Hristopulos
%   email: dchristopoulos@ece.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%   [y] = wls_weighted(Param, model, varioe, weighting)
%==========================================================================

function [y] = wls_weighted(Param, model, varioe, weighting)

lags = varioe.lags;
varioz = varioe.varioz; 
npair = varioe.npair;

if nargin < 4
    weighting=0;
end

modelnam = model.model;
dd = model.dd;

nparam=length(Param);
Params = Param(2:end); % parameters without nugget

if nparam >4
    disp('Wrong number of parameters');
    return;
end

switch modelnam 
    case 'SSRF'
        [S0] = cov_ssrf(dd, eps, Params, 0);
        [Cr] = cov_ssrf(dd, lags, Params, 0);
        variom = S0 - Cr;

    case 'Expo'
        variom = Param(2) *  ( 1 - exp(-lags/Param(3)) ); 
        
    case 'Gaus'
        variom = Param(2) *  ( 1 - exp(- (lags/Param(3)).^2) ); 

    case 'Sphe'
        h = lags/Param(3); 
        variom =  Param(2) * ( ( 3 *h/2  - (h.^3)/2 ).* (h <=1) + 1 .* (h>1) );
    case 'Mate'
        h = lags/Param(3); 
        nu = Param(4); 
        cb = 2^(1-nu)/gamma(nu);
        variom =  Param(2) * ( 1 -  cb * (h.^nu) .* besselk(nu, h) );    
    case 'Power'
        variom = Param(2) * (lags).^Param(3);
 
end

variom = variom + Param(1); 

if weighting==1 % Weight based on number of pairs 
    y = sum(npair.*(varioz-variom).^2);
elseif weighting==2  % Weight normalized by theoretical variogram 
     y = sum(  (npair ./ varioz) .*  ((varioz-variom).^2) );
else% No weights
    y=sum((varioz-variom).^2);
end



